import org.ccil.cowan.sax.*;
import org.xml.sax.*;
import org.xml.sax.helpers.*;
import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import java.net.*;
import java.util.*;


/** The LinkMap class creates a small imagemap applet that uses 
XML files for its source material.  In this version, it supports 
extended links in multiple directions, building a link list as it goes. */

public class LinkMap extends Applet implements ActionListener  {
	protected Image image;
	protected String caption;
	protected String xmlDoc;
	protected URL xmlURL;
	protected LinkSet links;
	protected LinkSet tempLinks;
	protected Vector areas;
	private EventHandler handler;
	private ContextPopup popup;

	//  Initialize and load first page
	public void init() {

		//links variable must be initialized because
		//it will be added to, not replaced.
		links=new LinkSet();
		xmlDoc=getParameter("src");
		try {
		  xmlURL=new URL(getDocumentBase(), xmlDoc);
		} catch  (Exception e) {
		   e.printStackTrace();
		}
		
		//  Load and Parse Document
		load (xmlURL);

		//create new context-sensitive popup menu
		popup=new ContextPopup(this);
		this.add(popup);
		this.enableEvents (AWTEvent.MOUSE_EVENT_MASK);
	}

	public void load (URL loadURL) {
		LinkSet tempLinks=new LinkSet();


		try {
		   handler=new EventHandler(loadURL, tempLinks);
		} catch (Exception e) {
		   e.printStackTrace();
		}
		
		//  get image 

		image=this.getImage(loadURL, handler.getImage());		

		//  note that links are _added_ on top of each other
		links.addLinkSet(tempLinks);

		//  areas are replaced
		areas=handler.getAreas();
		Area tempArea;
		for (int i=0; i<areas.size(); i++) {
		tempArea=(Area) areas.elementAt(i);

		}
		// change URL
		xmlURL=loadURL;
		//  caption is set
		caption=handler.getCaption();
		repaint();
	}
	public void processMouseEvent (MouseEvent e) {
		if (e.isPopupTrigger()) 
			popup.show(this, e.getX(), e.getY());
		else super.processMouseEvent (e);	
	}

	
	public void update(Graphics g) { paint(g); }

	public void paint(Graphics g) {
		g.drawImage(image, 0, 0, this);
		showStatus(caption);
	}

	public void actionPerformed(ActionEvent event) {
		//responds to the menu.  URLs are passed
		//as commands		

		String command = event.getActionCommand();
		if (!command.equals("No Link")) 
		{ try {
		    URL commandURL=new URL (command); 
		    load (commandURL);
		    } catch (Exception e) {
		     e.printStackTrace();
		    }//end try-catch
		}//end if
	}//end actionPerformed

/*  This class does a lot of heavy lifting. It determines what links to 
what, based on the areas and links between them.  It also allows for 
non-id-based connections between whole documents.  This avoids the need 
to specify areas on _every_ image and makes it easier to go back. */

class ContextPopup extends PopupMenu {
	private LinkMap parent;

	public ContextPopup(LinkMap parentApplet) {
		super();
		parent=parentApplet; //need this to get links and areas
	}

	public void show(Component pass, int x, int y){
		MenuItem mi;
		Area areaTemp;
		String baseUrl=parent.xmlURL.toString();
		String idTemp;
		String destUrlTemp;
		LinkSet linksTemp;
		Link linkTemp;

		//clear menu
		this.removeAll();
		for (int i=0; i<parent.areas.size(); i++) {
			areaTemp=(Area)parent.areas.elementAt(i);
			if (areaTemp.isInArea(x,y)) {
				//first check for explicit links from this
				//area to others

				idTemp=areaTemp.getId();
				linksTemp=parent.links.getByURLAndLoc(baseUrl, idTemp);
				for (int j=0; j<linksTemp.size(); j++){
				   linkTemp=linksTemp.linkAt(j);
				   createItem (linkTemp.getDestTitle(), linkTemp.getDestURL());
				}//end for that adds returned links to menu
			}//end area If
		}//end area For
		
		//then check for links that don't spec ID

		idTemp="%none";

		linksTemp=parent.links.getByURLAndLoc(baseUrl, idTemp);
		for (int k=0; k<linksTemp.size(); k++){
		     linkTemp=linksTemp.linkAt(k);
		     createItem (linkTemp.getDestTitle(), linkTemp.getDestURL());
		}//end for that adds returned links to menu
		
		//if no links
		if (this.getItemCount()==0)
			createItem ("No Link", "No Link");
				
		//finally, pass to superclass for display.
		super.show(pass, x, y);
	}//end method show
	
	//  Add Item to Menu
	private void createItem (String title, String url) {
		MenuItem mi = new MenuItem(title);
		mi.setActionCommand(url);
		mi.addActionListener(parent);
		this.add(mi);
	}//end method createItem
   }//end inner class ContextPopup
}//end class LinkMap